-- now that i've finished my first "reskinning" of the UI, i've 
-- decided that the skin scripts need some restructuring.
--
-- under the new pattern, a skin is a table of named functions.
-- each function takes a GUI environment as input, and returns
-- one or more tables as output.  for example:
--
-- local panel = myskin.FleetPanel(_ENV) 
--
-- it's expected that the skin functions will be called when
-- the base UI class is loaded (in this example, that means
-- when GalaxyMap.lua loads).  so if you need to change a skin
-- during runtime, the only way to do it is to change the
-- active skin, then force a reload of the UI file.  for
-- example, GalaxyMap.lua contains a function set_skin that
-- looks like this:
--
--  function set_skin(skin)
--   	myskin = gui[skin]
--		if not myskin
--			trace_error(format("%q is not a valid skin name",tostring(skin)))
--		end
--		reload_script([[GUI\GalaxyMap\GalaxyMap.lua]])
--	end
--
-- this is awkward -- but, allowing reskinning during runtime
-- isn't really a key feature; that it works at all is a nice
-- perk.  however, the benefits of sharing an environment
-- between the skin and the UI code are large.
--
-- most of the work involved in setting up any skinnable UI
-- element happens inside a resize callback; the tables
-- returned by the skin's function are often initially empty.
-- they'll be filled with useful functions when the UI's resize
-- callbacks are triggered.  as a result, you should only cache 
-- the functions inside a skin if you know that those caches
-- will be cleared the next time the resize callbacks are
-- triggered.

local fname = current_lua_file()
local skinname = current_lua_dir_leaf():sub(2)

_ENV[skinname] = _ENV[skinname] or {}