

UI_File | function(_ENV) 

create_leftside_pane 'planet_pop_pane'

local pane = planet_pop_pane!

local needs_rebuild=true
do

  --add_resize_callback | function() needs_rebuild=true end
  function after_weak_clears.reset_planet_pop_pane()
    needs_rebuild=true
  end

  local last_empire
  function pane.rebuild_if_needed()
    needs_rebuild = needs_rebuild or  (last_empire~=gui_player())
    if needs_rebuild
      pane.rebuild_with_old_scroll()
      needs_rebuild=false
      last_empire=gui_player()
    end
  end

end

local sorting_options = {
  'Population',
  'Max Population',
  'Habitability',
  'Planet Type',
  'Planet Size',
  'Unrest',
  'Name',

   on_select = function(key)
    needs_rebuild=pane.sort_by~=key
    pane.sort_by=key
   end,

   make_word = function(text)
     return pane.text_context.lazy_word_render( 
      (text==pane.sort_by) and'vera_bold16' or 'vera14',text)
   end,

   word_offset=function(text,i)
    return (text==pane.sort_by) and -1
   end,
   sel_y_pad=3,
}

pane.sort_by = pane.sort_by or 'Population'

ListFrame.create_pane(_ENV,|) {
  name = 'planet_pop_pane',
  title = 'Population Report',
  tiny_prefix_icon='people',
  tiny_prefix_dy=3,
}

planet_list_hover_text = make_hover_text_cache { 
  name = 'planet_list_hover_text',
  text_width = 350,
  bk_color = catch(0,0,0,.75),
  draw_style = 'draw_bl_bk',
  default_font_key = 'vera14',
}

local sort_funs = {
}

function sort_funs.Population (p1,p2)
  return lexical_compare(
    total_pop(p2), total_pop(p1),
    p1.name , p2.name,
    p1._ikey, p2._ikey)
 end

function sort_funs.Name (p1,p2)
  return lexical_compare(
    p1.name , p2.name,
    p2.outpost and 0 or total_pop(p2), p1.outpost and 0 or total_pop(p1),
    p1._ikey, p2._ikey)
 end

sort_funs['Max Population'] = function(p1,p2)
  return lexical_compare(
    p2:inhabited_max(), p1:inhabited_max(),
    p1.name , p2.name,
    p1._ikey, p2._ikey)
 end

-- sort in order of favourability to the current empire's species
local planet_habitat_metric = strong_memoizeNN | function(type, size, species)
  --local species = gui_player().species
  local affinity = PlanetTypes.affinities[species]
  local habitats = PlanetTypes.types[type][size]

  local score = 0
  for habitat,n ; habitats
    score += affinity[habitat]*n
  end
  return score
end

function sort_funs.Habitability(p1,p2)
  local species = gui_player().species
  return lexical_compare(
    planet_habitat_metric(p2.type, p2.size, species),
    planet_habitat_metric(p1.type, p1.size, species),
    p1.name , p2.name,
    p1._ikey, p2._ikey)
 end

--similar to Habitability sort, except planet-size independent
sort_funs['Planet Type'] = function(p1,p2)
  local species = gui_player().species
  return lexical_compare(
    planet_habitat_metric(p2.type, 'Medium', species),
    planet_habitat_metric(p1.type, 'Medium', species),
    p1.name , p2.name,
    p1._ikey, p2._ikey)
 end

local planet_sizes = {
  Tiny   = 1,
  Small  = 2,
  Medium = 3,
  Large  = 4,
  Huge   = 5,
}

sort_funs['Planet Size'] = function(p1,p2)
  return lexical_compare(
    planet_sizes[p2.size], planet_sizes[p1.size],
    p1.name , p2.name,
    p1._ikey, p2._ikey)
 end

function sort_funs.Unrest(p1,p2)
  return lexical_compare(
    p2:chance_to_revolt(), p1:chance_to_revolt(),
    p1.name , p2.name,
    p1._ikey, p2._ikey)
end

local get_planet_list = weak_memoizeNN | function (empire,sort_by)
  
  local push, list = pusher {}
  for p ; empire.planets
    push | p
  end

  --print(v2s | empire.planets)

  local sort_by in pane!
  table.sort(list, sort_funs![sort_by]) 

  return list
end

function select_next_planet(el)
  local planet in el!
  local list = get_planet_list ( planet.empire, pane.sort_by)


  for i=1,#list
    if planet==list[i]


      for k=0,#list
        local j= ( ( i+k) % #list ) + 1
        el.planet=list[j]
        if el.planet.productive
          break
        end
      end

      Sounds.WIDGET_INCREMENT_UP()
      center_at_star(el.planet.star)
      select_map_key(el.planet.star)
      return planet_info_click(el)
    end
  end
end

function select_prev_planet(el)
  local planet in el!
  local list = get_planet_list ( planet.empire, pane.sort_by)
  for i=1,#list
    if planet==list[i]

      for k=1,#list
        local j= i-k


        if j<1
          j= #list + j
        end

        el.planet=list[j]
        if el.planet.productive
          break
        end
      end

      Sounds.WIDGET_INCREMENT_DOWN()
      center_at_star(el.planet.star)
      select_map_key(el.planet.star)
      return planet_info_click(el)
    end
  end
end


add_resize_callback | function  ()

  local tooltip_el = new_tip_maker {
    style = 'draw_center_left', 
    no_desc_text=true,
    dx=52,
    dy=16,

   -- dx=10,
   -- dy=0,
  }
  local tp_element,tooltip_id = tooltip_el('Sort By:')

  local text_bk_color=drawers.frame_colors.dark_dropdown_color

  --local text_bk_color=catch(0,0,0,1)

  sorting_options.on_choosing_draw = function()
     tooltip_activations.mark(tooltip_id)
  end

  function pane.draw_overlay_content(x,y,w,h)
      vertical_arrows.draw_small_down( x+w-67, y+h-21 )

    local el = SCOPE_call(..text_bk_color, tp_element,x+w-100,y+h-34,46+20,38)
    local dropdown_id = sorting_options



    local rx,ry,rw,rh = x+w-85,y+h-34,153,38

    dropdown_menu.register_menu(dropdown_id, sorting_options,
      rx-20,ry,rw,rh)

    el.dropdown_id=dropdown_id
    el.on_click = dropdown_menu.on_open_dropdown
    --glBox()
  end

end

function planet_pop_pane.rebuild_vbox_list(vbox_list)

  local plus_form in _ENV

  local w in vbox_list
  local item_h=62
  local test_bk = restore_color | function ()
    set_frame_color4(1,1,1,.3)
    --glBox(0,0,w,item_h)
  end

  --print 'rebuilding list!'

  vbox_list.clear() 

  local lazy_word,lazy_word_hcentered,lazy_word_render in vbox_list!.text_rendering_context

  vbox_list.add_vspace(8)

  local x1=60
  local y1=item_h-20

  local size_to_s = {
		Tiny=.5,
    Small=.65,
    Medium=.75,
    Large=.88,
    Huge=1,
  }

  local make_number = strong_memoize_1_to_1 | function (i)
    
    local pt,font in fonts.orig21

    pt=max(14,round | pt*current_gl_scale())

--      print(..pt)

    local text,info = outlined_text {
      center = catch(1,1,1,1),
      outline1 = catch(0,0,0,.3),
      outline_width1=4,
      ..font,
      ..pt,
      text = tostring(i),
    }


    return info
  end

  local function planet_icon_request(planet,x0,y0)
    local s=size_to_s[planet.size]
    local max_s=28
    local x,y,w,h = box_grow( s*max_s,s*max_s, 30,30, 0,0 )

    if not options.hide_planet_icons
      --print(..y0,..y)
      request_planet(planet,x0+x,y0+y ,w)

--       execute_draw_requests()
    end   
  end

  local planet_draw = safe_requests | function(planet)
     
    local s=size_to_s[planet.size]
    local max_s=28
    local x,y,w,h = box_grow( s*max_s,s*max_s, 30,30, 0,0 )

    local word = lazy_word_render(
      'orig21',
      planet.name,
      x1,y1
    )
    word.draw(x1,y1-14)

    local word2 = lazy_word_render(
      'orig12',
      planet.size.." "..planet.type,
      x1,y1
    )
    word2.draw(x1,y1-28)

    if planet.productive

      local orender = make_number(display_pop(planet) )

      local image in orender
      local iw,ih = image:dims()

      local x,y = screen_position(30,y1-12)

      x,y-=iw*.5,ih*.5

    	call_with_identity_matrix(image.draw, image, round(x), round(y))

    end

    return word
  end

  local shadow_options = {sigma=.8,spread=.5,opacity=.9}
  local small_shadowed_word_drawer  = weak_memoizeNN | function(text)
    local tooltip_text_context in SCOPE._ELEMENT_ENV
     local word= tooltip_text_context!.lazy_word_render('vera10',text)
      if not word.draw_shadow
      shadowed_word(shadow_options,word)
    end

    return function(cx,cy)
      local x=cx-word.w*.5
      local y=cy
      word.draw_shadow(x,y)
      word.draw(x,y)
    end
  end

  local draw_population = restore_color | function (planet)

    if planet.inhabited
      local pop_w = 20
      local eee_offset= vbox_list.w - 80 - pop_w*nkeys(planet.pop_and_tanks)

      for race,p in pairs(planet.pop_and_tanks)
        drawers.PopIcons.eee_chibi_draw(race, x1+eee_offset+pop_w/2, 15)
        small_shadowed_word_drawer(format("%d", round | p))(x1+eee_offset+pop_w/2, 5)

        eee_offset += pop_w
      end

    end

  end

  local function list_planet_click(el)
    select_map_key(el.planet.star)
    planet_click(el)
    center_at_star(el.planet.star)
  end

  local function list_transport_click(el)
    center_at_star(el.planet.star)
    on_planet_transport_button(el)

  end

  for i,planet in ipairs | get_planet_list ( gui_player(), pane.sort_by)

    if not planet.inhabited
      goto next_planet
    end

    --
    -- i'm definitely getting a bit overly creative with shared
    -- upvalues here.
    --
    -- the use of "title" and "hover_id" both take advantage of the
    -- assumption that each draw cycle, 'draw' will be called
    -- before 'draw_hover'.  while it works fine, and is actually
    -- fairly safe -- it's also potentially pretty damn confusion.
    -- um, sorry about that ;-P
    --
    local transport_tooltip_id = {}
    local draw 
    local title_word
    local hover_id


    local draw_hover_buttons = SCOPE_fun(icon_radius_hint=12,|) function (hover_id,title_word)
       set_frame_color4(1,1,1,1)

       local x = x1+title_word.w+5
--       glBox(x,y1-2,18,18)
      -- ResourceGlyph.transport.draw_glyph(18,x,y1-2)


      local cx,cy = x+12,y1+8-18

      local text,can_transport = PlanetTransport.get_planet_tooltip(planet)

      local icon = can_transport and 'spaceship_colored' or 'spaceship_grey'

      local el = element | notification_icons.misc[icon].draw(cx,cy)
      el.hover_id = draw
      el.tooltip_id = transport_tooltip_id
      el.on_click = can_transport and list_transport_click
      el.planet=planet

    end

    local draw_button_tooltips = SCOPE_fun(text_bk_color=catch(0,0,0,.7),|) function (hover_id,title_word)
      set_frame_color4(1,1,1,1)
       local x = x1+title_word.w+5
      local cx,cy = x+12,y1+8
      local tooltip_id = transport_tooltip_id
      local text,can_transport = PlanetTransport.get_planet_tooltip(planet)
      draw_tooltip_cl(tooltip_id,cx+20,cy,text)

    end


    local function button_draw_fun(fun)
       if not planet_selection_mode

        local hover_id = draw
        local hover_alpha = hover_alpha_states[hover_id] or 0
        if hover_alpha > 0
          call_in_color_scope(SCOPE_frame_activation, hover_alpha, fun,hover_id,title_word)
        end
      end     
    end


    draw = restore_color | function()

      hover_id = draw
      
      if (midscreen_pane == embark_pane) or (planet_selection_mode==PlanetTransport)
        hover_id= 'TT' .. planet._recorded_name
      end

      insert(element(0,0,w,item_h),
        on_hover = hand_cursor,
        on_click = list_planet_click,
        transport_key=planet,
        ..planet,
        map_key=planet.star,
        ..hover_id
      )
      --test_bk()

      title_word = planet_draw(planet)
      draw_population(planet)


      button_draw_fun(draw_hover_buttons)
    end




    hover_id_to_speed[draw]=4
    
    --SCOPE_call('list_icon', true
    
    local _,vbox = vbox_list.add_vbox (draw,item_h)

    function vbox.draw_hover()

      button_draw_fun(draw_button_tooltips)

      
      if hover_alpha_states[hover_id] and is_transport_hover_id(hover_id )

        local hover_text = get_transport_hover_text(planet)
        set_frame_color4(1,1,1,1)
        if hover_text

          local cx,cy=x,y
          planet_list_hover_text.hover_text(hover_id ,40,9,
            hover_text
          --  'hi\nlets add\nmultiple lines'
          )

        end
      end
  
    end

    --[[
    vbox.draw=restore_color | function(x,y)
      print('draw',x,y)
      glColor4d(0,1,0,.6)
      glBox(0,0,100,5)
    end

    vbox.draw_hover=restore_color | function(x,y)
      print('draw_hover',x,y)
      glColor4d(0,0,1,.6)
      glBox(0,0,100,5)
    end
    --]]

    function vbox.request_bk_content(x,y) 
      planet_icon_request(planet,x,y)
    end


    vbox_list.add_vspace(4)

    ::next_planet::

  end

end


end

if lua_load_complete and false



  options.hide_planet_icons=false

end

reload_script [[GUI\~GalaxyMap\GalaxyMap.lua]] 